unit Header;

interface

uses
  SysUtils, WinTypes, WinProcs, Classes;

type
  { Enter 'Delay': off, as checkbox or as edit field }
  TDelayed = (dOff, dCheck, dEdit);

  { Enum for labels (CKeys and CLabelValues) }
  KeyIndexes = (
    KIQueryFrom, KIWaiting, KIOptions,
    KIClose, KIAbout, KIPhoneBook,
    KIReceiver, KIFaxNums, KIRecvName,
    KISender, KIUser, KISendName, KIAlias,
    KIOK, KICancel, KIPort, KILabels, KILoadNetAPI, KISaveHist,
    KIHistory, KIQuerySave, KIMore, KILookupHost, KIMoreEnabled,
    KIMaxHistory, KIErrReadOnlyEntry,
    KIDelayed, KIdOff, KIdCheck, KIdEdit
  );

  TChar = array[0..255]of Char;

const
  DefaultPort = 5555;

  KIFirst = KIQueryFrom;       { First element of KeyIndexes }
  KILast = KIdEdit;            { Last element of KeyIndexes  }

  CProgName = 'Respond';       { Program name }
  CProgVersion = '1.6preview3a';{ Version      }

  { INI file names }
  { Respond.ini for aliases and history }
  IniFile : array[0..255]of Char = CProgName + '.ini';
{  IniName: PChar = @IniFile;}
  { Win.ini for options }
  WinIniFile: Array[0..255]of Char = 'WIN.INI';
  WinIni: PChar = @WinIniFile;
  { new: File for options }
  OptionsFile: TChar = '';
  { ini file for labels, regularly Win.ini }
  LabelFile: array[0..255]of Char = 'WIN.INI';
  LabelIni: PChar = @WinIniFile;
  { file for private phonebook entries, regularly Win.ini }
  PrivatePhonebookFile: TChar = '';
  PrivatePhonebookIni: PChar = @IniFile;
  { file for history entries, regularly Win.ini }
  HistoryFile: TChar = '';
  HistoryIni: PChar = @IniFile;
  { Not used yet: file for common (shared) read only phonebook entries}
  CommonPhonebookFile: TChar = '';
  CommonPhonebookIni: PChar = @CommonPhonebookFile;

  { Menuitems for system menu or trayicon popup menu }
{$IFDEF RESPOND95}
  CloseMenuItemString : array[0..255]of Char = '';
  CloseMenuItem : PChar = @CloseMenuItemString;
{$ENDIF}
  AboutMenuItemString : array[0..255]of Char = '';
  AboutMenuItem : PChar = @AboutMenuItemString;
  PBookMenuItemString : array[0..255]of Char = '';
  PBookMenuItem : PChar = @PBookMenuItemString;

{$IFDEF RESPOND95}
  { Registry entry for user name }
  UserNameKey: PChar = 'Network\Logon';
  UserNameValue: PChar = 'username';
{$ENDIF}

  { WIN.INI entries: }
  { Section names }
  OptionsSection : PChar = CProgName;
  LabelsSection : PChar = CProgName+'-Labels';
  { Key names }
  PortValueKey : PChar = 'Port';
  DelayedKey : PChar = 'Delayed';
  LoadNetAPIKey: PChar = 'LoadNetAPI';
  SaveHistoryKey: PChar = 'SaveHistory';
  MaxHistKey: PChar = 'MaxHistory';
  QueryReceiverNameKey: PChar = 'QueryReceiverName';
  QueryUserKey: PChar = 'QueryUser';
  QuerySenderNameKey: PChar = 'QuerySenderName';
  RememberMoreKey: PChar = 'RememberMore';
  LookupHostKey: PChar = 'LookupHostName';
  MoreKey: PChar = 'MoreEnabled';

  { Respond.ini entries: }
  AliasSection : PChar = 'Aliases';
  HistorySection : PChar = 'History';

  KMaxHistory = 10;
  KGetMem = 2048;

  KeyCount = Ord(KILast)-Ord(KIFirst)+1;       { Label key count }

  { CKeys covers the label keys from WIN.INI }
  CKeys : array[KeyIndexes]of PChar = (
    'QueryFrom',
    'Waiting',
    'Options',
    'Close',
    'About',
    'PhoneBook',
    'Receiver',
    'Faxnumbers',
    'ReceiverName',
    'Sender',
    'User',
    'SenderName',
    'Alias',
    'OK',
    'Cancel',
    'Port',
    'Labels',
    'LoadNetAPI',
    'SaveHistory',
    'History',
    'QuerySave',
    'More',
    'LookupHostName',
    'MoreEnabled',
    'MaxHistory',
    'ErrReadOnlyEntry',
    'Delayed',
    'DelayedOff',
    'DelayedCheck',
    'DelayedEdit'
  );

  { CLabelValues covers the label names from WIN.INI }
  CLabelValues : array[KeyIndexes]of PChar = (
    'Query from',
    'Waiting',
    '&Options',
    '&Close',
    '&About',
    '&Phonebook',
    'Receiver',
    '&Fax-Number(s)',
    '&Name',
    'Sender',
    '&User',
    'N&ame',
    'A&lias',
    '&OK',
    '&Cancel',
    '&Port',
    '&Labels',
    'Load N&ETAPI.DLL',
    'Save &History',
    'History',
    'Save changed values?',
    'Mo&re',
    '&Lookup host name',
    'Additional input &field',
    '&Max. history entries',
    'This entry is read only!',
    '&Delayed Send',
    'Off',
    'Checkbox',
    'Editfield'
  );

procedure ProcessOptions;
function UnShortCut( const s: String ): String;
function GetLabelStr(const KI: KeyIndexes): String;
procedure GetLabelPChar(const KI: KeyIndexes; p: PChar; l: Integer);

implementation

{ Process commandline options }
procedure ProcessOptions;
type
  { Enumeration of existing options }
  OptionType = (oLabels,      { Labels, win.ini }
                oOptions,     { Options, win.ini }
                oAliases,     { Private phonebook, respond.ini }
                oHistory,     { History, respond.ini }
                oPrivate,     { depricated, options and labels, win.ini }
                oCommonPB,    { Common read only phonebook, none }
                oOldLabels);  { depricated, labels, win.ini }
const
  OptionCount = 7;
  { Array of existing options }
  OptionChars: array [1..OptionCount] of Char = ('b', 'o', 'a', 'h', 'p', 'c', 'l');
  { Array of enumerations sets of existing options }
  OptionSets: array [1..OptionCount] of
                set of OptionType = ([oLabels], [oOptions], [oAliases],
                                     [oHistory], [oPrivate], [oCommonPB], [oOldLabels]);
var
  o, fn: String;
  i, m, n: Integer;
  os: set of OptionType;
begin
  for i := 1 to ParamCount do begin
    o := ParamStr(i);
    { If it is a switch }
    if (o[1] = '-') or (o[1] = '/') then begin
      { Delete switch char }
      Delete(o,1,1);
      { Clear options set }
      os := [];
      { Search options string for every existing options char }
      for n := 1 to OptionCount do begin
        m := Pos(OptionChars[n], o);
        { if found }
        if m > 0 then begin
          { add it to options set }
          os := os + OptionSets[n];
          { old style options processing: }
          { Put labels in Respond.ini }
          if o[m] = 'l' then LabelIni := @IniFile;
          { Put labels and options in Respond.ini }
          if o[m] = 'p' then begin
            WinIni := @IniFile;
            LabelIni := @IniFile;
          end;
          { delete found char from options string }
          delete(o, m, 1);
        end;
      end;
    end else begin
      { Make sure that dir strings ends with \ }
      if o[Length(o)] <> '\' then AppendStr(o, '\');
      { Build filename }
      fn := o+CProgName+'.ini';
      { Set filename for every option in options set
       (i.e. options found in option string) }
      if oLabels in os then begin
        StrPCopy(LabelFile, fn);
        LabelIni := @LabelFile;
      end;
      if oOptions in os then begin
        StrPCopy(OptionsFile, fn);
        WinIni := @OptionsFile;
      end;
      if oAliases in os then begin
        StrPCopy(PrivatePhonebookFile, fn);
        PrivatePhonebookIni := @PrivatePhonebookFile;
      end;
      if oHistory in os then begin
        StrPCopy(HistoryFile, fn);
        HistoryIni := @HistoryFile;
      end;
      if oCommonPB in os then begin
        StrPCopy(CommonPhonebookFile, fn);
        CommonPhonebookIni := @CommonPhonebookFile;
      end;
      { Old style option processing }
      if (oPrivate in os) or
         (oOldLabels in os) then StrPCopy(IniFile, fn);
      { Clear optios set; needed in case of invalid commandline }
      os := [];
    end;
  end;
end;

function UnShortCut( const s: String ): String;
{ Remove Shortcut-'&' from String }
var
  i: Integer;
begin
  result := s;
  i := Pos( '&', result );
  if i > 0 then Delete( result, i, 1 );
end;

function GetLabelStr(const KI: KeyIndexes): String;
{ Read label from INI file as String }
var
  p : array[0..255]of Char;
begin
  GetLabelPChar(KI, @p, 255);
  Result := StrPas( @p );
end;

procedure GetLabelPChar(const KI: KeyIndexes; p: PChar; l: Integer);
{ Read label from INI file as PChar }
begin
  GetPrivateProfileString( LabelsSection, CKeys[KI], CLabelValues[KI],
                           p, l, LabelIni );
end;

end.
