unit Phbook;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids, Header;

const
  KGetMem = 2048;

type
  { Phonebook entry }
  TAlias = record
    FaxNum, Empfaenger: String;
    Common: Boolean;
  end;
  PAlias = ^TAlias;

  TPhoneBook = class(TStringList)
  public
    constructor Create;
    procedure Read(PhonebookFile: PChar; ReadOnly: Boolean);
    procedure Update(PhonebookFile: PChar);
  end;

  TPhBookForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    AliasList: TListBox;
    PlusBtn: TButton;
    MinusBtn: TButton;
    Alias: TEdit;
    FaxNum: TEdit;
    Empfaenger: TEdit;
    EmpfaengerLabel: TLabel;
    FaxNumLabel: TLabel;
    AliasLabel: TLabel;
    AliasListLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure AliasListClick(Sender: TObject);
    procedure PlusBtnClick(Sender: TObject);
    procedure MinusBtnClick(Sender: TObject);
    procedure AliasChange(Sender: TObject);
    procedure AliasListExit(Sender: TObject);
    procedure AliasListEnter(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  PhBookForm: TPhBookForm;
  PhoneBook, CommonPhoneBook: TPhoneBook;

procedure CreatePBookForm(AOwner: TForm);
procedure ExecutePBookForm;

implementation

{$R *.DFM}

const
  CommonMarker = '-';

constructor TPhoneBook.Create;
begin
  inherited Create;
  Sorted := True;
  Duplicates := dupIgnore;
end;

procedure TPhoneBook.Read(PhonebookFile: PChar; ReadOnly: Boolean);
{ Read phonebook from ini file }
var
  i : Integer;
  p : array[0..255]of Char;
  p2, p3 : PChar;
  a : PAlias;
  s : String;
begin
  Getmem( p2, KGetMem );
  { Read aliases from phonebook }
  GetPrivateProfileString( AliasSection, nil, '', p2, KGetMem, PhonebookFile );
  p3 := p2;
  { Get data for every alias }
  while p3^ <> #0 do begin
    { Read value }
    GetPrivateProfileString( AliasSection, p3, '', @p, 255, PhonebookFile );
    s := StrPas( @p );
    i := Pos( ':', s );
    { Create new phonebook entry }
    New(a);
    { Split value into number and name }
    a^.FaxNum := Copy( s, 1, i-1 );
    a^.Empfaenger := Copy( s, i+2, length(s) );
    a^.Common := ReadOnly;
    { Add entry to phonebook }
    try
      AddObject( StrPas( p3 ), TObject(a) );
    except
      { Throw away entry if not enough memory }
      on EOutOfMemory do Dispose(a);
    end;
    { Goto next alias }
    p3 := StrEnd( p3 );
    Inc(p3);
  end;
  FreeMem( p2, KGetMem );
end;

procedure TPhoneBook.Update(PhonebookFile: PChar);
{ Write phonebook to ini file }
var
  i : Integer;
  p, p1 : array[0..255]of Char;
  a : PAlias;
begin
  if PhBookForm = nil then exit;
  for i := 0 to Count-1 do begin
    a := PAlias(Objects[i]);
    if PhBookForm.AliasList.Items.IndexOf(Strings[i]) < 0 then begin
      { remove alias key from ini file if it was deleted }
      StrPCopy( @p, Strings[i] );
      WritePrivateProfileString( AliasSection, @p, nil, PhonebookFile );
    end;
    Dispose( a );
  end;
  Clear;  { Clear phonebook... }
  with PhBookForm.AliasList do for i := 0 to Items.Count-1 do begin
    { ...and copy all entries from form alias list to phonebook... }
    Self.AddObject( Items.Strings[i], Items.Objects[i] );
    StrPCopy( @p, Items.Strings[i] );
    StrPCopy( @p1, PAlias(Items.Objects[i])^.FaxNum+': '+
                  PAlias(Items.Objects[i])^.Empfaenger );
    { ...and write to ini file }
    WritePrivateProfileString( AliasSection, @p, @p1, PhonebookFile );
  end;
  { Flush ini file }
  WritePrivateProfileString( nil, nil, nil, PhonebookFile );
end;

procedure TPhBookForm.FormCreate(Sender: TObject);
begin
  Caption := CProgName+' '+UnShortCut(StrPas(PBookMenuItem));
  OKBtn.Caption := GetLabelStr(KIOK);
  CancelBtn.Caption := GetLabelStr(KICancel);
  AliasLabel.Caption := GetLabelStr(KIAlias);
  AliasListLabel.Caption := GetLabelStr(KIAlias);
  FaxNumLabel.Caption := GetLabelStr(KIFaxNums);
  EmpfaengerLabel.Caption := GetLabelStr(KIRecvName);
end;

procedure TPhBookForm.AliasListClick(Sender: TObject);
{ Replacement for the OnChange list event }
var
  a: PAlias;
  s: String;
begin
  with AliasList do begin
    if ItemIndex >= 0 then begin
      a := PAlias(Items.Objects[ItemIndex]);
      s := Items.Strings[ItemIndex];
      { The second condition shouldn't be necessary }
      if (a^.Common) and (Pos(CommonMarker,s) = 1) then Delete(s,1,Length(CommonMarker));
      Alias.Text := s;
      FaxNum.Text := a^.FaxNum;
      Empfaenger.Text := a^.Empfaenger;
      PlusBtn.Default := False;
      OKBtn.Default := False;
      MinusBtn.Default := True;
    end;
  end;
end;

procedure TPhBookForm.PlusBtnClick(Sender: TObject);
{ Add an alias }
var
  a: PAlias;
  i: Integer;
begin
  if ( Alias.Text <> '' ) and ( FaxNum.Text <> '' ) then begin
    with AliasList do begin
      i := Items.IndexOf( AnsiLowerCase(Alias.Text) );
      if i >= 0 then begin
        a := PAlias(Items.Objects[i]);
        if a^.Common then begin
          MessageDlg(GetLabelStr(KIErrReadOnlyEntry), mtError, [mbOK], 0);
          exit;
        end else begin
          Dispose( PAlias(Items.Objects[i]) );
          Items.Delete( i );
        end;
      end;
      i := Items.IndexOf( CommonMarker+AnsiLowerCase(Alias.Text) );
      if i >= 0 then begin
        a := PAlias(Items.Objects[i]);
        if a^.Common then begin
          MessageDlg(GetLabelStr(KIErrReadOnlyEntry), mtError, [mbOK], 0);
          exit;
        end;
      end;
      try
        New(a);
        a^.FaxNum := FaxNum.Text;
        a^.Empfaenger := Empfaenger.Text;
        a^.Common := False;
      except
        on EOutOfMemory do exit;
      end;
      try
        Items.AddObject( AnsiLowerCase(Alias.Text), TObject(a) );
        Alias.Text := '';
        FaxNum.Text := '';
        Empfaenger.Text := '';
      except
        on EOutOfMemory do Dispose(a);
      end;
    end;
    Alias.SetFocus;
  end;
end;

procedure TPhBookForm.MinusBtnClick(Sender: TObject);
{ Remove an alias }
var
  i: Integer;
  a : PAlias;
begin
  with AliasList do begin
    for i := 0 to Items.Count-1 do
    if Selected[i] then begin
      a := PAlias(Items.Objects[i]);
      if a^.Common then begin
        MessageDlg(GetLabelStr(KIErrReadOnlyEntry), mtError, [mbOK], 0);
      end else begin
        Alias.Text := Items.Strings[i];
        FaxNum.Text := a^.FaxNum;
        Empfaenger.Text := a^.Empfaenger;
        Dispose(a);
        Items.Delete( i );
        break;
      end;
    end;
    if Items.Count > 0 then
      SetFocus
    else
      Alias.SetFocus;
  end;
end;

procedure CreatePBookForm(AOwner: TForm);
{ Create phonebook from, set labels and fill alias list;
  don't ShowModal it now, to allow presetting of some edit fields }
var
  i : Integer;
  a : PAlias;
begin
  PhBookForm := TPhBookForm.Create(AOwner);
  for i := 0 to PhoneBook.Count-1 do begin
    New(a);
    a^ := PAlias(PhoneBook.Objects[i])^;
    try
      if a^.Common then
        PhBookForm.AliasList.Items.AddObject( CommonMarker+PhoneBook.Strings[i], TObject(a) )
      else
        PhBookForm.AliasList.Items.AddObject( PhoneBook.Strings[i], TObject(a) );
    except
      on EOutOfMemory do Dispose(a);
    end;
  end;
end;

procedure ExecutePBookForm;
{ ShowModal the prepared phonebook form }
var
  i : Integer;
begin
  with PhBookForm do begin
    if ShowModal = IDOK then PhoneBook.Update(PrivatePhonebookIni)
    else begin
      for i := 0 to AliasList.Items.Count-1 do
        Dispose( PAlias(AliasList.Items.Objects[i]) );
    end;
  end;
  PhBookForm.Free;
end;

procedure TPhBookForm.AliasChange(Sender: TObject);
begin
  if not AliasList.Focused then begin
    PlusBtn.Enabled := Alias.Text <> '';
    PlusBtn.Default := PlusBtn.Enabled;
    OKBtn.Default := not PlusBtn.Default;
  end;
end;

procedure TPhBookForm.AliasListExit(Sender: TObject);
begin
  if MinusBtn.Default = True then begin
    MinusBtn.Default := False;
    if Alias.Text <> '' then PlusBtn.Default := True
    else OKBtn.Default := True;
  end;
end;

procedure TPhBookForm.AliasListEnter(Sender: TObject);
begin
  if AliasList.ItemIndex >= 0 then begin
    PlusBtn.Default := False;
    OKBtn.Default := False;
    MinusBtn.Default := True;
  end;
end;

end.
