unit Rootfrm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus,
{$IFDEF RESPOND95}
  TrayIcon,
{$ENDIF}
  BuffSock, DWinSock, WsAPI,
  Respfrm, Header, PhBook, Histfrm, AboutFrm, Morefrm;

type
  TRootForm = class(TForm)
    SSocket: TTextServer;
    { No IFDEF because of automatic remove by Delphi 1 }
    TrayIcon: TTrayIcon;
{$IFNDEF RESPOND95}
    procedure wmSysCommand( var Msg : TMsg; var Handled : Boolean );
{$ENDIF}
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure SSocketAccept(Sender: TObject; Socket: TSocketBase);
    procedure SSocketDisconnect(Sender: TObject; Socket: TSocketBase);
  private
    { Private-Deklarationen }
{$IFNDEF RESPOND95}
    Appended : Boolean;         {Systemmenu extended}
    SysMenu : HMenu;
    NetApi : THandle;
{$ENDIF}
    wartend, angenommen : String;
    Accepted : Boolean;         {Communication is running}
    CSocket : TLineSocket;          {Active socket}
{$IFDEF RESPOND95}
    procedure MenuClose( Sender: TObject );
    procedure AppMinimize(Sender: TObject);
{$ENDIF}
    procedure WMQueryOpen(VAR Msg : TWMQueryOpen); message WM_QUERYOPEN;
    procedure MenuAbout( Sender: TObject );
    procedure MenuPBook(Sender: TObject);
public
    { Public-Deklarationen }
    procedure MainFormOKBtnClick(Sender: TObject);
    procedure MainFormCancelBtnClick(Sender: TObject);
  end;

var
  RootForm: TRootForm;

implementation

{$IFNDEF RESPOND95}
const
  { wmSysCommand: wParam-values for system menu items
                  last 4 Bits have to be 0 (reserved for Windows }
  SC_ABOUT = $0020;
  SC_PBOOK = $0030;

type
  TNetWkStaGetInfo = function( pszServer : PChar; sLevel : Integer;
                               pbBuffer : PChar; cbBuffer : Word;
                               pcbTotalAvail : PWord) : Word;

var
  NetWkStaGetInfo : TNetWkStaGetInfo;
  LibLoaded : Boolean;               { NETAPI.DLL loaded }
{$ENDIF}

{$R *.DFM}

function GetUserName : String;
{$IFDEF RESPOND95}
const
  DBuf : array[0..255]of char = '';
var
  DType : DWORD;
  BufSiz : DWORD;
  KeyHandle: HKEY;
begin
  BufSiz := 255;
  WNetGetUser(nil, @DBuf, BufSiz);  { Get the local username }
  if StrLen(@DBuf) = 0 then begin
    BufSiz := 255;
    { Ask registry for username }
    RegOpenKeyEx( HKEY_LOCAL_MACHINE, UserNameKey, 0, KEY_QUERY_VALUE, KeyHandle );
    RegQueryValueEx( KeyHandle, UserNameValue, nil, @DType, @DBuf, @BufSiz );
    RegCloseKey( KeyHandle );
  end;
  Result := StrPas( @DBuf );
{$ELSE}
{ Use NETAPI.DLL to get the user name, read SYSTEM.INI if this fails }
type
  TWkSta_info_10 = record
    wki_computername : PChar;	{ name of computer }
    wki_username : PChar;	{ name of logged on user}
    wki_langroup : PChar;	{ name of workgroup}
    wki_ver_major : Byte;	{ major version number}
    wki_ver_minor : Byte;	{ minor version number }
    wki_logon_domain : PChar;	{ name of logon workgroup}
    wki_oth_domains : PChar;	{ reserved; must be zero  }
  end;

var
  TotalSize, Res, Size : Word;
  Wki : TWkSta_info_10;
  p : array[0..255]of char;
Begin
  Result := '';
  if LibLoaded then   { only if NETAPI.DLL loaded }
    if NetWkstaGetInfo(nil,10,@wki,SizeOf(Wki),@TotalSize) = 0 then
      Result := StrPas(wki.wki_UserName);
  if Result = '' then begin
    GetPrivateProfileString( 'Network', 'UserName', '', @p, 255, 'SYSTEM.INI' );
    Result := StrPas( @p );
  end;
{$ENDIF}
end;

procedure TRootForm.SSocketAccept(Sender: TObject;
  Socket: TSocketBase);
{ a host calls us }
var
  s : String;
begin
  If not Accepted then begin
    Accepted := True;
    WritePrivateProfileString(nil, nil, nil, WinIni);
    if WinIni <> LabelIni then
      WritePrivateProfileString(nil, nil, nil, LabelIni);
    { prepare Respond window }
    MainForm := TMainForm.Create(Self);
    s := '';
    if GetPrivateProfileInt(OptionsSection, LookupHostKey, 0, WinIni) <> 0 then
      s := Socket.RemoteHost;
    if s = '' then s := Socket.RemoteAddress;
    Caption := CProgName;
{$IFDEF RESPOND95}
    TrayIcon.ToolTip := CProgName;
{$ENDIF}
    CSocket := Socket as TLineSocket;
    with MainForm do begin
      Caption := angenommen+s;
      if Nutzer.Visible then
        Nutzer.Text := GetUserName;
      OKBtn.OnClick := MainFormOKBtnClick;
      CancelBtn.OnClick := MainFormCancelBtnClick;
      Show;
{$IFNDEF RESPOND95}
      { Needed for 16bit version with win95 to show window above all other }
      SetWindowPos( handle, HWND_TOPMOST, Left, Top, Width, Height,
                    SWP_NOSIZE or SWP_NOMOVE );
      { and now remove TOPMOST status }
      SetWindowPos( handle, HWND_TOP, Left, Top, Width, Height,
                    SWP_NOSIZE or SWP_NOMOVE );
{$ELSE}
      SetForegroundWindow( handle );
{$ENDIF}
      Eingabe.SetFocus;
      Eingabe.SelectAll;
    end;
  end else Socket.Close;      { allow only one connection at the same time }
end;

procedure TRootForm.SSocketDisconnect(Sender: TObject;
  Socket: TSocketBase);
{ Remote host has closed connection }
begin
  MainForm.Close;
  MainForm.Free;
  Socket.Close;
  Caption := wartend;
{$IFDEF RESPOND95}
  TrayIcon.ToolTip := wartend;
{$ENDIF}
  Accepted := False;
end;

{$IFDEF RESPOND95}
procedure TRootForm.AppMinimize(Sender: TObject);
begin
  ShowWindow(Application.Handle, SW_HIDE);
end;
{$ENDIF}

procedure TRootForm.WMQueryOpen(VAR Msg : TWMQueryOpen);
{ prevent window open }
begin
  Msg.Result := 0
end;

procedure TRootForm.FormCreate(Sender: TObject);
var
{  s : String;
  i : Integer;}
{$IFNDEF RESPOND95}
  ProcAddress : TFarProc;
{$ELSE}
  MenuItem : TMenuItem;
{$ENDIF}
begin
{$IFDEF RESPOND95}
  ShowWindow( Handle, SW_HIDE );
  Application.OnMinimize := AppMinimize;
{$ENDIF}
{
  for i := 1 to ParamCount do begin
    s := ParamStr(i);
    if (s = '-l') or (s = '/l') then LabelIni := @IniFile
    else if (s = '-p') or (s = '/p') then begin
      WinIni := @IniFile;
      LabelIni := @IniFile;
    end else begin
      if s[Length(s)] <> '\' then AppendStr(s, '\');
      StrPCopy(@IniFile, s+CProgName+'.ini');
    end;
  end;
}
  { Process comandline options }
  ProcessOptions;
  PhBookForm := nil;
  AboutBox := nil;
  PhoneBook := nil;
  HistoryList := nil;
  MoreStrings := nil;
  PhoneBook := TPhoneBook.Create;
  PhoneBook.Read(PrivatePhonebookIni, False);
  CommonPhoneBook := TPhoneBook.Create;
  CommonPhoneBook.Read(CommonPhoneBookIni, True);
  HistoryList := THistory.Create;
  HistoryList.Read;
  Accepted := False;
  GetLabelPChar(KIAbout, @AboutMenuItemString, 255 );
  GetLabelPChar(KIPhoneBook, @PBookMenuItemString, 255 );
  angenommen := CProgName+': '+GetLabelStr(KIQueryFrom)+' ';
  wartend := CProgName+' ('+GetLabelStr(KIWaiting)+')';
  Caption := wartend;
{$IFNDEF RESPOND95}
  {Try to load NETAPI.DLL}
  LibLoaded := False;
  { but only if it is allowed (prevent error message in Win31/WinNT which
    do not have one) }
  if ( GetPrivateProfileInt( OptionsSection, LoadNetAPIKey, 0, WinIni ) <> 0 ) then begin
    NetAPI := LoadLibrary( 'NETAPI.DLL' );        { load NETAPI.DLL dynamically }
    if not ( NetAPI < HINSTANCE_ERROR) then begin
      ProcAddress := getprocaddress( NetAPI, 'NetWkStaGetInfo' );
                                     { Get address of NetWkStaGetInfo }
      if ProcAddress = nil then FreeLibrary( NetAPI )
                                     { Free DLL if function not found }
      else begin
        NetWkStaGetInfo := TNetWkStaGetInfo( ProcAddress );
        LibLoaded := True;           { DLL loaded and function found}
      end;
    end;
  end;
{$ENDIF}
{$IFNDEF RESPOND95}
  { Change system menu }
  Appended := False;
  Application.OnMessage := wmSysCommand;
  SysMenu := GetSystemMenu( Handle, False );
  AppendMenu( SysMenu, MF_SEPARATOR, 0, '' );
  AppendMenu( SysMenu, MF_STRING, SC_ABOUT, AboutMenuItem );
  AppendMenu( SysMenu, MF_STRING, SC_PBOOK, PBookMenuItem );
  Appended := True;
{$ELSE}
  { Create PopupMenu }
  TrayIcon.PopupMenu := TPopupMenu.Create(Self);
  GetLabelPChar(KIClose, @CloseMenuItemString, 255 );
  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := StrPas(CloseMenuItem);
  MenuItem.OnClick := MenuClose;
  TrayIcon.PopupMenu.Items.Add(MenuItem);
  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := StrPas(AboutMenuItem);
  MenuItem.OnClick := MenuAbout;
  TrayIcon.PopupMenu.Items.Add(MenuItem);
  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := StrPas(PBookMenuItem);
  MenuItem.OnClick := MenuPBook;
  TrayIcon.PopupMenu.Items.Add(MenuItem);
  TrayIcon.ToolTip := Caption;
  TrayIcon.Active := True;
{$ENDIF}
  { Listen }
  SSocket.Port := IntToStr(GetPrivateProfileInt( OptionsSection, PortValueKey, DefaultPort, WinIni ));
  try
    SSocket.Listen( TLineSocket );
  except
    on E:ESockError do begin
      MessageDlg( E.Message, mtError, [mbOK], 0 );
      Application.Terminate;
    end;
  end;
end;

procedure TRootForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SSocket.CloseDown;  { Don't listen any more }
{$IFNDEF RESPOND95}
  SysMenu := GetSystemMenu( Handle, True );  { Restore system menu }
{$ELSE}
  TrayIcon.Active := False;
{$ENDIF}
{$IFNDEF RESPOND95}
  if LibLoaded then FreeLibrary( NetAPI );   { Free NETAPI.DLL }
{$ENDIF}
  HistoryList.Write;
  Action := caFree;                          { Free all form memory }
end;

procedure TRootForm.FormDestroy(Sender: TObject);
var
  i : Integer;
begin
{$IFDEF RESPOND95}
  TrayIcon.PopupMenu.Free;
{$ENDIF}
  if HistoryList <> nil then begin
    for i := HistoryList.Count-1 downto 0 do StrDispose( PChar(HistoryList.Objects[i]) );
    HistoryList.Free;
  end;
  if PhoneBook <> nil then begin
    for i := PhoneBook.Count-1 downto 0 do Dispose( PAlias(PhoneBook.Objects[i]) );
    PhoneBook.Free;
  end;
  if MoreStrings <> nil then begin
    MoreStrings.Clear;
    MoreStrings.Free;
    MoreStrings := nil;
  end;
end;

{$IFNDEF RESPOND95}
procedure TRootForm.wmSysCommand( var Msg : TMsg; var Handled : Boolean );
{ Process system menu commands }
begin
  Handled := False;
  if Msg.Message = WM_SYSCOMMAND then begin
    case ( Msg.wParam and $FFF0 ) of
    SC_ABOUT : begin
        { Show aboutbox }
        MenuAbout(Self);
        Handled := True;   { Command processed }
      end;
    SC_PBOOK : begin
        { Show phonebook }
        MenuPBook(Self);
        Handled := True;   { Command processed }
      end;
    end;
  end;
end;

{$ELSE}

procedure TRootForm.MenuClose( Sender: TObject );
begin
  {Application.Terminate;}
  Close;
end;
{$ENDIF}

procedure TRootForm.MenuAbout( Sender: TObject );
{ ShowModal aboutbox }
begin
  AboutBox := TAboutBox.Create( Self );
  AboutBox.ShowModal;
  AboutBox.Free;
end;

procedure TRootForm.MenuPBook(Sender: TObject);
{ ShowModal phonebook }
begin
  CreatePBookForm(Self);
  ExecutePBookForm;
end;

procedure TRootForm.MainFormOKBtnClick(Sender: TObject);
{ OK button in query form clicked; send input }
var
  i : Integer;
  s, s1, s3 : String;
  p : array[0..255]of Char;
  p1, p2 : PChar;
  a : PAlias;
begin
  if Accepted then with MainForm do begin
    StrPCopy( @p, Eingabe.Text );
    p1 := @p;
    s := '';
    s3 := '';
    repeat
      while (( p1^=' ' ) or ( p1^=',' )) and ( p1^<>#0 ) do Inc(p1);
      p2 := p1;
      while ( p1^<>' ' ) and ( p1^<>',' ) and ( p1^<>#0 ) do Inc(p1);
      if p1^ <> #0 then begin
        p1^ := #0;
        Inc(p1);
      end;
      if p2^ <> #0 then begin
        i := PhoneBook.IndexOf( StrPas(p2) );
        if i >= 0 then begin
          a := PAlias(PhoneBook.Objects[i]);
          if Empfaenger.Text = '' then
            if s = '' then s3 := a^.Empfaenger
            else s3 := '';
          s1 := a^.FaxNum;
        end else s1 := StrPas(p2);
        AppendStr(s, ' '+s1);
      end;
    until p1^ = #0;
    if s <> '' then while s[1] = ' ' do Delete( s, 1, 1 );
    if s <> '' then begin
      if Empfaenger.Text = '' then Empfaenger.Text := s3;
      HistoryList.AddEntry( s, Empfaenger.Text );
      CSocket.WriteLine(s);
      if Nutzer.Visible then CSocket.WriteLine(Nutzer.Text)
                        else CSocket.WriteLine('');
      if Empfaenger.Visible then CSocket.WriteLine(Empfaenger.Text)
                        else CSocket.WriteLine('');
      if Name.Visible then CSocket.WriteLine(Name.Text)
                        else CSocket.WriteLine('');
      if Delayed <> dOff then begin
        case Delayed of
        dCheck: begin
            if DelayedCheck.Checked then CSocket.WriteLine('delayed');
    {        DelayedCheck.Checked := False;}
          end;
        dEdit: begin
            CSocket.WriteLine(DelayedEdit.Text);
    {        DelayedEdit.Text := '';}
          end;
        else CSocket.WriteLine('');
        end;
      end else CSocket.WriteLine('');
      if (MoreStrings <> nil) and (MoreStrings.Count > 0) then begin
        for i := 0 to MoreStrings.Count-1 do
          CSocket.WriteLine(MoreStrings[i]);
      end;
      CSocket.Close;    { Close connection after sending }
      Close;
      Release;
      if GetPrivateProfileInt( OptionsSection, RememberMoreKey, 0, WinIni ) = 0 then begin
        MoreStrings.Free;
        MoreStrings := nil;
      end;
      Caption := wartend;
      Accepted := False;
    end else Eingabe.SetFocus;
  end;
end;

procedure TRootForm.MainFormCancelBtnClick(Sender: TObject);
{ Cancel button in query form clicked; close connection }
begin
  if Accepted then CSocket.Close;
   MainForm.Close;
   MainForm.Release;
   if GetPrivateProfileInt( OptionsSection, RememberMoreKey, 0, WinIni ) = 0 then begin
     MoreStrings.Free;
     MoreStrings := nil;
   end;
   Caption := wartend;
   Accepted := False;
 end;

end.
